%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Analyze the Impulse Response
% 20 December 2006, Release 1.0
%
% Input:
%
% Output:
%
% Internal Variables:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load required MAT files
load ChanNum
cd(strcat('CM',num2str(ChanNum)))
load ImpResp
cd ..

% do some screen formatting
disp(' '); disp(' '); disp(' '); disp(' '); disp(' '); disp(' '); disp(' ')
disp(' '); disp(' '); disp(' '); disp(' '); disp(' ')

%********************************************************************
% Testing and ploting of Channel Metrics
%********************************************************************

% determine channel energy
kk=0;
for ChanIndex=1:NumChannels
    if NothingLeft(ChanIndex)==0
        kk=kk+1; % artifical counter so nulled channels don't matter
        [Row,Col]=size(ImpDtTrunc);
        if Row>1
            channel_energy(kk) = sum(abs(ImpDtTrunc(:,ChanIndex)).^2); 
        else
            channel_energy(kk) = abs(ImpDtTrunc(ChanIndex)).^2; 
        end
    end
end

% calculate metrics
excess_delay = zeros(1,NumChannels);
RMS_delay = zeros(1,NumChannels);
num_sig_paths = zeros(1,NumChannels);
num_sig_e_paths = zeros(1,NumChannels);

kk=0;
for k=1:NumChannels
    if NothingLeft(k)==0
       kk=kk+1; % artifical counter so nulled channels don't matter
       % determine excess delay and RMS delay
       sq_h = abs(ImpDtTrunc(:,k)).^2 / channel_energy(kk);
       t_norm = TimeDt - t0(k);  % remove the randomized arrival time of first cluster
       excess_delay(kk) = t_norm * sq_h;
       RMS_delay(kk) = sqrt( ((t_norm-excess_delay(kk)).^2) * sq_h );

       % determine number of significant paths (paths within 10 dB from peak)
       threshold_dB = -10;   % dB
       temp_h = abs(ImpDtTrunc(:,k));
       temp_thresh = 10^(threshold_dB/20) * max(temp_h);
       num_sig_paths(kk) = sum(temp_h > temp_thresh);

       % determine number of sig. paths (captures x % of energy in channel)
       x = 0.85;
       temp_sort = sort(temp_h.^2);  % sorted in ascending order of energy
       cum_energy = cumsum(temp_sort(end:-1:1));  % cumulative energy
       index_e = min(find(cum_energy >= x * cum_energy(end)));
       num_sig_e_paths(kk) = index_e;
    end
end

energy_mean = mean(10*log10(channel_energy));
energy_stddev = std(10*log10(channel_energy));
mean_excess_delay = mean(excess_delay);
mean_RMS_delay = mean(RMS_delay);
mean_sig_paths = mean(num_sig_paths);
mean_sig_e_paths = mean(num_sig_e_paths);

% print this to the screen
disp(' '); fprintf(1,'Model Characteristics\n');
fprintf(1,'  Mean delays: excess (tau_m) = %1.2f ns, RMS (tau_rms) = %1.2f ns\n', ...
    1e9*mean_excess_delay, 1e9*mean_RMS_delay);
fprintf(1,'  # paths: NP_10dB =  %.1f, NP_85%% = %.1f\n', ...
    mean_sig_paths, mean_sig_e_paths);
fprintf(1,'  Channel energy: mean = %.1f dB, std deviation = %.1f dB\n', ...
  energy_mean, energy_stddev);
fprintf(1,'  Channels Spatially Nulled: %.1f , Remaining Channels: %.1f\n', ...
  sum(NothingLeft), NumChannels-sum(NothingLeft));

% plot out the statistics
figure(1)
MaxImp=max(max(abs(ImpDtTrunc)));
plot(TimeDt,20*log10(1e-2+abs(ImpDtTrunc/MaxImp))); grid on % limit the lower log value
axis tight
title('Impulse Response dB Magnitude')
xlabel('nS')
ylabel('dB')

figure(2)
subplot(2,1,1); plot(TimeDt,real(ImpDtTrunc)); title('real impulse response'); grid on
subplot(2,1,2); plot(TimeDt,imag(ImpDtTrunc)); title('imag impulse response'); grid on
xlabel('nS')

figure(3); clf; plot(1:NumChannels, excess_delay, 'b-', ...
  [1 NumChannels], mean_excess_delay*[1 1], 'r--' );
grid on
title('Excess delay (nS)')
xlabel('Channel number')

figure(4); clf; plot(1:NumChannels, RMS_delay, 'b-', ...
  [1 NumChannels], mean_RMS_delay*[1 1], 'r--' );
grid on
title('RMS delay (nS)')
xlabel('Channel number')

figure(5); clf; plot(1:NumChannels, num_sig_paths, 'b-', ...
  [1 NumChannels], mean_sig_paths*[1 1], 'r--');
grid on
title('Number of significant paths within 10 dB of peak')
xlabel('Channel number')

figure(6); clf; plot(1:NumChannels, num_sig_e_paths, 'b-', ...
  [1 NumChannels], mean_sig_e_paths*[1 1], 'r--');
grid on
title('Number of significant paths capturing > 85% energy')
xlabel('Channel number')

temp_average_power = sum((abs(ImpDtTrunc))'.*(abs(ImpDtTrunc))')/NumChannels;
temp_average_power = temp_average_power/max(temp_average_power);
average_decay_profile_dB = 10*log10(temp_average_power+1e-20);
figure(7); clf; plot(TimeDt,average_decay_profile_dB); grid on
axis([0 TimeDt(end) -60 0])
title('Average Power Decay Profile')
xlabel('Delay (nsec)')
ylabel('Average power (dB)')

figure(8)
stem(NothingLeft,'.')
title('Nulled Channels due to Spatial Filtering')
xlabel('Channel Number')
ylabel('Nulled Channel')
grid on

% query user to view a particular impulse response
disp(' ')
Question = -1; % initialize to illegal value
while (Question>2 || Question<1)
    Question = input('Do you want to view a realization Impulse Response? [1=NO, 2=YES] ');
end

if Question==2
    disp(strcat('Maximum Number of Realizations:_',num2str(NumChannels)))
    CHAN = -1; % initialize to illegal value
    while (CHAN>NumChannels || CHAN<1)
        CHAN = input('Input Realization Number to View: ');
    end
    figure(9)
    subplot(2,1,1); plot(1e9*TimeDt,real(ImpDtTrunc(:,CHAN)))
    title('real impulse response'); grid on
    subplot(2,1,2); plot(1e9*TimeDt,imag(ImpDtTrunc(:,CHAN)))
    title('imag impulse response'); grid on
    xlabel('nS')

    figure(10)
    plot(1e9*TimeDt,abs(ImpDtTrunc(:,CHAN)))
    title('mag impulse response'); grid on
    xlabel('nS')

end

return