%----------------------------------------------------------------------------------
%
% Description :
% This function M-file outputs values of electric strength according to 
% AOAs of rays
% The antenna models contributed in TG3C can be used:
% Reference antenna model (IEEE 15-06-0427-04-003c)
% Gaussian-distributed antenna model (IEEE 15-06-0195-03-003c)
%
% Programmed by R. Funada, H. Sawada, K. Iigusa, and H. Harada
%----------------------------------------------------------------------------------

function [g, fai] = tsv_ant_gain_r6(ant_type, hpbw, fai0, azimuth, fig_on, TITLE)
% Arguments
% ant_type	: Antenna model used in simulation
%	1: Reference antenna model
%	2: Gaussian-distributed antenna model
% hpbw		: Half-power angle in deg
% fai0		: Angle of arrival in deg
% azimuth	: Azimuth in deg
% Option
%	fig_on	: Index of figure that shows relative antenna gain
%	TITLE	: figure title
% Output value
% g		: Electric strength (True value)

if azimuth ~=0
	ray_aoa_azi	= exp(j.*(fai0+azimuth)./180*pi);
	fai = angle(ray_aoa_azi)/pi*180;
else
	fai= fai0;
end

switch ant_type
	case 1 %Reference antenna model
		G0 = 10*log10((1.6162./sin(hpbw*pi/180/2))^2);
		theta_ml=2.6*hpbw;
		G_side = -0.4111.*log(hpbw)-10.597;
		G = G0 - 3.01 .* (2.*abs(fai)./hpbw).^2;

		pos=find(abs(fai)>theta_ml/2);
		G(pos)=G_side;
		g0=G-G0;
		g = 10.^(g0/20);
	case 2 %Gaussian-distributed antenna model
		alfa = 4*log(2)./(hpbw*pi/180).^2;
		g	= sqrt(exp(-alfa.*abs(fai./180*pi).^2));
	otherwise
		error('Antenna model error')
end

if nargin >= 5
	figure(fig_on)
	set(gca,'FontName','Arial','FontSize',12)
	plot(fai,20*log10(g))
	xlabel('Angle [deg]')
	ylabel('Relative antenna gain [dB]')
end

if nargin >= 6
	title(TITLE)
end